@@ -12,6 +12,7 @@ related utilities used by hledger commands.
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell     #-}
 {-# LANGUAGE TypeFamilies        #-}
+{-# LANGUAGE TypeOperators       #-}
 
 module Hledger.Cli.CliOptions (
 
@@ -37,6 +38,7 @@ module Hledger.Cli.CliOptions (
   withAliases,
   likelyExecutablesInPath,
   hledgerExecutablesInPath,
+  ensureDebugHasArg,
 
   -- * CLI options
   CliOpts(..),
@@ -57,7 +59,7 @@ module Hledger.Cli.CliOptions (
   outputFileFromOpts,
   outputFormatFromOpts,
   defaultWidth,
-  widthFromOpts,
+  -- widthFromOpts,
   replaceNumericFlags,
   -- | For register:
   registerWidthsFromOpts,
@@ -77,15 +79,15 @@ import Control.Monad (when)
 import Data.Char
 import Data.Default
 import Data.Either (fromRight, isRight)
-import Data.Functor.Identity (Identity)
 import Data.List.Extra (groupSortOn, intercalate, isInfixOf, nubSort)
-import Data.List.Split (splitOneOf)
+import Data.List.Split (splitOn)
 import Data.Maybe
 --import Data.String.Here
 -- import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Void (Void)
 import Safe
+import String.ANSI
 import System.Console.CmdArgs hiding (Default,def)
 import System.Console.CmdArgs.Explicit
 import System.Console.CmdArgs.Text
@@ -96,6 +98,7 @@ import System.Directory
 import System.Environment
 import System.Exit (exitSuccess)
 import System.FilePath
+import System.Info (os)
 import Text.Megaparsec
 import Text.Megaparsec.Char
 
@@ -103,17 +106,19 @@ import Hledger
 import Hledger.Cli.DocFiles
 import Hledger.Cli.Version
 import Data.Time.Clock.POSIX (POSIXTime)
+import Data.List (isPrefixOf, isSuffixOf)
 
 
 -- common cmdargs flags
+-- keep synced with flag docs in doc/common.m4
 
 -- | Common help flags: --help, --debug, --version...
 helpflags :: [Flag RawOpts]
 helpflags = [
   -- XXX why are these duplicated in defCommandMode below ?
   flagNone ["help","h"] (setboolopt "help") "show general help (or after CMD, command help)"
- ,flagNone ["man"] (setboolopt "man") "Show user manual with man"
- ,flagNone ["info"] (setboolopt "info") "Show info manual with info"
+ ,flagNone ["man"] (setboolopt "man") "show user manual with man"
+ ,flagNone ["info"] (setboolopt "info") "show info manual with info"
  -- ,flagNone ["browse-args"] (setboolopt "browse-args") "use a web UI to select options and build up a command line"
  ,flagReq  ["debug"] (\s opts -> Right $ setopt "debug" s opts) "[N]" "show debug output (levels 1-9, default: 1)"
  ,flagNone ["version"] (setboolopt "version") "show version information"
@@ -159,7 +164,7 @@ reportflags = [
  ,flagReq  ["depth"]         (\s opts -> Right $ setopt "depth" s opts) "NUM" "(or -NUM): hide accounts/postings deeper than this"
  ,flagNone ["empty","E"]     (setboolopt "empty") "show items with zero amount, normally hidden (and vice-versa in hledger-ui/hledger-web)"
 
-  -- valuation
+  -- valuation, including https://hledger.org/dev/hledger.html#valuation-type :
  ,flagNone ["B","cost"]      (setboolopt "B")
    "show amounts converted to their cost/selling amount, using the transaction price."
  ,flagNone ["V","market"]    (setboolopt "V")
@@ -182,8 +187,9 @@ reportflags = [
      ,"YYYY-MM-DD: convert to market value on the given date, in default valuation commodity or COMM"
      ])
   ,flagNone ["infer-equity"] (setboolopt "infer-equity")
-    "in conversion transactions, replace costs (transaction prices) with equity postings, to keep the transactions balanced"
-  
+    "infer conversion equity postings from costs"
+  ,flagNone ["infer-costs"] (setboolopt "infer-costs")
+    "infer costs from conversion equity postings"
   -- history of this flag so far, lest we be confused:
   --  originally --infer-value
   --  2021-02 --infer-market-price added, --infer-value deprecated 
@@ -195,16 +201,18 @@ reportflags = [
   --    --costs deprecated and hidden, uses --infer-market-prices instead
   --    --inverted-costs renamed to --infer-reverse-prices
  ,flagNone ["infer-market-prices"] (setboolopt "infer-market-prices") 
-    "use transaction prices (recorded with @ or @@) as additional market prices, as if they were P directives"
+    "use costs as additional market prices, as if they were P directives"
 
-  -- generated postings/transactions
- ,flagNone ["auto"]          (setboolopt "auto") "apply automated posting rules to modify transactions"
- ,flagOpt "" ["forecast"]    (\s opts -> Right $ setopt "forecast" s opts) "PERIODEXP" 
-  (unlines
-   [ "Generate periodic transactions (from periodic transaction rules). By default these begin after the latest recorded transaction, and end 6 months from today, or at the report end date."
-   , "Also, in hledger-ui, make future transactions visible."
-   , "Note that = (and not a space) is required before PERIODEXP if you wish to supply it."
+  -- generating transactions/postings
+ ,flagOpt "" ["forecast"]    (\s opts -> Right $ setopt "forecast" s opts) "PERIOD" (unwords
+   [ "Generate transactions from periodic rules,"
+   , "between the latest recorded txn and 6 months from today,"
+   , "or during the specified PERIOD (= is required)."
+   , "Auto posting rules will be applied to these transactions as well."
+   , "Also, in hledger-ui make future-dated transactions visible."
    ])
+ ,flagNone ["auto"]          (setboolopt "auto") "Generate extra postings by applying auto posting rules to all txns (not just forecast txns)."
+ ,flagNone ["verbose-tags"]  (setboolopt "verbose-tags") "Add visible tags indicating transactions or postings which have been generated/modified."
 
   -- general output-related
  ,flagReq ["commodity-style", "c"] (\s opts -> Right $ setopt "commodity-style" s opts) "COMM"
@@ -220,7 +228,7 @@ reportflags = [
      ,"A NO_COLOR environment variable overrides this."
      ])
  ,flagOpt "yes" ["pretty"] (\s opts -> Right $ setopt "pretty" s opts) "WHEN"
-   (unlines
+   (unwords
      ["Show prettier output, e.g. using unicode box-drawing characters."
      ,"Accepts 'yes' (the default) or 'no'."
      ,"If you provide an argument you must use '=', e.g. '--pretty=yes'."
@@ -321,7 +329,7 @@ defCommandMode names = defMode {
 -- given name, providing hledger's common input/reporting/help flags.
 -- Just used when invoking addons.
 addonCommandMode :: Name -> Mode RawOpts
-addonCommandMode name = (defCommandMode [name]) {
+addonCommandMode nam = (defCommandMode [nam]) {
    modeHelp = ""
      -- XXX not needed ?
      -- fromMaybe "" $ lookup (stripAddonExtension name) [
@@ -367,34 +375,61 @@ hledgerCommandMode doc unnamedflaggroup namedflaggroups hiddenflaggroup argsdesc
         ,modeArgs        = argsdescr
         }
 
--- | Parse a command's documentation, as follows:
+-- | Parse a command's help text file (Somecommand.txt).
+-- This is generated from the command's doc source file (Somecommand.md)
+-- by Shake cmdhelp, and it should be formatted as follows:
 --
--- - First line: the command name then any aliases, as one or more space or comma-separated words
+-- - First line: main command name
 --
--- - Second line to a line containing just _FLAGS, or the end: the short help
+-- - Third line: command aliases, comma-and-space separated, in parentheses (optional)
 --
--- - Any lines after _FLAGS: the long help (split into lines for cmdargs)
+-- - Fifth or third line to the line containing just _FLAGS (or end of file): short command help
 --
--- The CLI help displays the short help, then the cmdargs-generated
--- flags list, then the long help (which some day we might make
--- optional again).  The manual displays the short help followed by
--- the long help, with no flags list.
+-- - Any lines after _FLAGS: long command help
+--
+-- The CLI --help displays the short help, the flags help generated by cmdargs,
+-- then the long help (which some day we might make optional again).
+-- The manual displays the short help, then the long help (but not the flags list).
 --
 parseCommandDoc :: CommandDoc -> Maybe ([Name], String, [String])
 parseCommandDoc t =
   case lines t of
     [] -> Nothing
-    (l:ls) -> Just (names, shorthelp, longhelplines)
+    (l1:_:l3:ls) -> Just (cmdname:cmdaliases, shorthelp, longhelplines)
       where
-        names = words $ map (\c -> if c `elem` [',','\\'] then ' ' else c) l
-        (shorthelpls, longhelpls) = break (== "_FLAGS") ls
+        cmdname = strip l1
+        (cmdaliases, rest) =
+          if "(" `isPrefixOf` l3 && ")" `isSuffixOf` l3
+          then (words $ filter (/=',') $ drop 1 $ init l3, ls)
+          else ([], l3:ls)
+        (shorthelpls, longhelpls) = break (== "_FLAGS") $ dropWhile (=="") rest
         shorthelp = unlines $ reverse $ dropWhile null $ reverse shorthelpls
         longhelplines = dropWhile null $ drop 1 longhelpls
+    _ -> Nothing  -- error' "misformatted command help text file"
 
 -- | Get a mode's usage message as a nicely wrapped string.
 showModeUsage :: Mode a -> String
-showModeUsage = (showText defaultWrap :: [Text] -> String) .
-               (helpText [] HelpFormatDefault :: Mode a -> [Text])
+showModeUsage =
+  highlightHelp .
+  (showText defaultWrap :: [Text] -> String) .
+  (helpText [] HelpFormatDefault :: Mode a -> [Text])
+
+-- | Add some ANSI decoration to cmdargs' help output.
+highlightHelp
+  | not useColorOnStdout = id
+  | otherwise = unlines . zipWith (curry f) [1..] . lines
+  where
+    f (n,s)
+      | n==1 = bold s
+      | s `elem` [
+           "General input flags:"
+          ,"General reporting flags:"
+          ,"General help flags:"
+          ,"Flags:"
+          ,"General flags:"
+          ,"Examples:"
+          ] = bold s
+      | otherwise = s
 
 -- | Get the most appropriate documentation topic for a mode.
 -- Currently, that is either the hledger, hledger-ui or hledger-web
@@ -486,8 +521,8 @@ rawOptsToCliOpts rawopts = do
   currentDay <- getCurrentDay
   let day = case maybestringopt "today" rawopts of
               Nothing -> currentDay
-              Just d  -> fromRight (error' $ "Unable to parse date \"" ++ d ++ "\"") -- PARTIAL:
-                         $ fixSmartDateStrEither' currentDay (T.pack d)
+              Just d  -> fromRight (error' $ "Unable to parse date \"" ++ d ++ "\"") $ -- PARTIAL:
+                         fromEFDay <$> fixSmartDateStrEither' currentDay (T.pack d)
   let iopts = rawOptsToInputOpts day rawopts
   rspec <- either error' pure $ rawOptsToReportSpec day rawopts  -- PARTIAL:
   mcolumns <- readMay <$> getEnvSafe "COLUMNS"
@@ -537,12 +572,12 @@ rawOptsToCliOpts rawopts = do
 -- add a space character to preserve them.
 --
 getHledgerCliOpts' :: Mode RawOpts -> [String] -> IO CliOpts
-getHledgerCliOpts' mode' args' = do
-  let rawopts = either usageError id $ process mode' args'
+getHledgerCliOpts' mode' args0 = do
+  let rawopts = either usageError id $ process mode' args0
   opts <- rawOptsToCliOpts rawopts
-  debugArgs args' opts
-  when ("help" `inRawOpts` rawopts_ opts) $ putStr shorthelp >> exitSuccess
-  -- when ("help" `inRawOpts` rawopts_ opts) $ putStr longhelp  >> exitSuccess
+  debugArgs args0 opts
+  when (boolopt "help" $ rawopts_ opts) $ putStr shorthelp >> exitSuccess
+  -- when (boolopt "help" $ rawopts_ opts) $ putStr longhelp  >> exitSuccess
   return opts
   where
     longhelp = showModeUsage mode'
@@ -554,12 +589,13 @@ getHledgerCliOpts' mode' args' = do
         ,"  See also hledger -h for general hledger options."
         ]
     -- | Print debug info about arguments and options if --debug is present.
+    -- XXX use standard dbg helpers
     debugArgs :: [String] -> CliOpts -> IO ()
-    debugArgs args' opts =
-      when ("--debug" `elem` args') $ do
+    debugArgs args1 opts =
+      when ("--debug" `elem` args1) $ do
         progname' <- getProgName
         putStrLn $ "running: " ++ progname'
-        putStrLn $ "raw args: " ++ show args'
+        putStrLn $ "raw args: " ++ show args1
         putStrLn $ "processed opts:\n" ++ show opts
         putStrLn $ "search query: " ++ show (_rsQuery $ reportspec_ opts)
 
@@ -588,7 +624,7 @@ expandPathPreservingPrefix d prefixedf = do
   let (p,f) = splitReaderPrefix prefixedf
   f' <- expandPath d f
   return $ case p of
-    Just p  -> p ++ ":" ++ f'
+    Just p'  -> p' ++ ":" ++ f'
     Nothing -> f'
 
 -- | Get the expanded, absolute output file path specified by an
@@ -633,16 +669,16 @@ rulesFilePathFromOpts opts = do
   d <- getCurrentDirectory
   maybe (return Nothing) (fmap Just . expandPath d) $ mrules_file_ $ inputopts_ opts
 
--- | Get the width in characters to use for console output.
--- This comes from the --width option, or the COLUMNS environment
--- variable, or (on posix platforms) the current terminal width, or 80.
--- Will raise a parse error for a malformed --width argument.
-widthFromOpts :: CliOpts -> Int
-widthFromOpts CliOpts{width_=Nothing, available_width_=w} = w
-widthFromOpts CliOpts{width_=Just s}  =
-    case runParser (read `fmap` some digitChar <* eof :: ParsecT Void String Identity Int) "(unknown)" s of
-        Left e   -> usageError $ "could not parse width option: "++show e
-        Right w  -> w
+-- -- | Get the width in characters to use for console output.
+-- -- This comes from the --width option, or the COLUMNS environment
+-- -- variable, or (on posix platforms) the current terminal width, or 80.
+-- -- Will raise a parse error for a malformed --width argument.
+-- widthFromOpts :: CliOpts -> Int
+-- widthFromOpts CliOpts{width_=Nothing, available_width_=w} = w
+-- widthFromOpts CliOpts{width_=Just s}  =
+--     case runParser (read `fmap` some digitChar <* eof :: ParsecT Void String Identity Int) "(unknown)" s of
+--         Left e   -> usageError $ "could not parse width option: "++errorBundlePretty e
+--         Right w  -> w
 
 -- for register:
 
@@ -659,7 +695,7 @@ registerWidthsFromOpts :: CliOpts -> (Int, Maybe Int)
 registerWidthsFromOpts CliOpts{width_=Nothing, available_width_=w} = (w, Nothing)
 registerWidthsFromOpts CliOpts{width_=Just s}  =
     case runParser registerwidthp "(unknown)" s of
-        Left e   -> usageError $ "could not parse width option: "++show e
+        Left e   -> usageError $ "could not parse width option: "++errorBundlePretty e
         Right ws -> ws
     where
         registerwidthp :: (Stream s, Char ~ Token s) => ParsecT Void s m (Int, Maybe Int)
@@ -705,21 +741,6 @@ dropRedundantSourceVersion fs = fs
 
 compiledExts = ["",".com",".exe"]
 
-
--- | Get all sorted unique filenames in the current user's PATH.
--- We do not currently filter out non-file objects or files without execute permission.
-likelyExecutablesInPath :: IO [String]
-likelyExecutablesInPath = do
-  pathdirs <- splitOneOf "[:;]" `fmap` getEnvSafe "PATH"
-  pathfiles <- concat `fmap` mapM getDirectoryContentsSafe pathdirs
-  return $ nubSort pathfiles
-  -- exclude directories and files without execute permission.
-  -- These will do a stat for each hledger-*, probably ok.
-  -- But they need paths, not just filenames
-  -- exes'  <- filterM doesFileExist exe'
-  -- exes'' <- filterM isExecutable exes'
-  -- return exes''
-
 -- | Get the sorted unique filenames of all hledger-* executables in
 -- the current user's PATH. These are files in any of the PATH directories,
 -- named hledger-*, with either no extension (and no periods in the name)
@@ -728,7 +749,25 @@ likelyExecutablesInPath = do
 hledgerExecutablesInPath :: IO [String]
 hledgerExecutablesInPath = filter isHledgerExeName <$> likelyExecutablesInPath
 
--- isExecutable f = getPermissions f >>= (return . executable)
+-- None of https://hackage.haskell.org/package/directory-1.3.8.1/docs/System-Directory.html#g:5
+-- do quite what we need (find all the executables in PATH with a filename prefix).
+-- | Get all sorted unique filenames in the current user's PATH.
+-- We do not currently filter out non-file objects or files without execute permission.
+likelyExecutablesInPath :: IO [String]
+likelyExecutablesInPath = do
+  pathdirs <- splitOn pathsep `fmap` getEnvSafe "PATH"
+  pathfiles <- concat `fmap` mapM getDirectoryContentsSafe pathdirs
+  return $ nubSort pathfiles
+  where pathsep = if os == "mingw32" then ";" else ":"
+--
+-- Exclude directories and files without execute permission:
+-- this would do a stat for each hledger-* file found, which is probably ok.
+-- But it needs file paths, not just file names.
+--
+-- exes'  <- filterM doesFileExist exe'
+-- exes'' <- filterM isExecutable exes'
+-- return exes''
+-- where isExecutable f = getPermissions f >>= (return . executable)
 
 isHledgerExeName :: String -> Bool
 isHledgerExeName = isRight . parsewith hledgerexenamep . T.pack
@@ -749,7 +788,10 @@ addonExtensions =
   ,"com"
   ,"exe"
   ,"hs"
+  ,"js"
   ,"lhs"
+  ,"lua"
+  ,"php"
   ,"pl"
   ,"py"
   ,"rb"
@@ -795,3 +837,10 @@ instance HasReportOptsNoUpdate CliOpts where
 
 instance HasReportOpts CliOpts where
     reportOpts = reportSpec.reportOpts
+
+-- | Convert an argument-less --debug flag to --debug=1 in the given arguments list.
+-- Used by hledger/ui/web to make their command line parsing easier somehow.
+ensureDebugHasArg as = case break (=="--debug") as of
+  (bs,"--debug":c:cs) | null c || not (all isDigit c) -> bs++"--debug=1":c:cs
+  (bs,["--debug"])                                    -> bs++["--debug=1"]
+  _                                                   -> as
