@@ -3,17 +3,24 @@ hledger's built-in commands, and helpers for printing the commands list.
 
 New built-in commands should be added in four places below:
 the export list, the import list, builtinCommands, commandsList.
+
 -}
 
+-- Note: commands list rendering is intensely sensitive to change,
+-- very easy to break in ways that tests currently do not catch.
+
+{-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE QuasiQuotes #-}
 {-# LANGUAGE TemplateHaskell #-}
 
 module Hledger.Cli.Commands (
-   findCommand
-  ,testcmd
+   testcmd
   ,builtinCommands
   ,builtinCommandNames
+  ,findBuiltinCommand
+  ,knownAddonCommands
+  ,knownCommands
   ,printCommandsList
   ,tests_Hledger_Cli
   ,module Hledger.Cli.Commands.Accounts
@@ -27,6 +34,7 @@ module Hledger.Cli.Commands (
   ,module Hledger.Cli.Commands.Close
   ,module Hledger.Cli.Commands.Codes
   ,module Hledger.Cli.Commands.Commodities
+  ,module Hledger.Cli.Commands.Demo
   ,module Hledger.Cli.Commands.Descriptions
   ,module Hledger.Cli.Commands.Diff
   ,module Hledger.Cli.Commands.Help
@@ -36,20 +44,20 @@ module Hledger.Cli.Commands (
   ,module Hledger.Cli.Commands.Payees
   ,module Hledger.Cli.Commands.Prices
   ,module Hledger.Cli.Commands.Print
-  ,module Hledger.Cli.Commands.Printunique
   ,module Hledger.Cli.Commands.Register
-  ,module Hledger.Cli.Commands.Registermatch
   ,module Hledger.Cli.Commands.Rewrite
   ,module Hledger.Cli.Commands.Stats
   ,module Hledger.Cli.Commands.Tags
 ) 
 where
 
-import Data.Char (isSpace)
+import Data.Char (isAlphaNum, isSpace)
 import Data.List
+import Data.List.Extra (nubSort)
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar
+import String.ANSI
 import System.Environment (withArgs)
 import System.Console.CmdArgs.Explicit as C
 import Test.Tasty (defaultMain)
@@ -69,6 +77,7 @@ import Hledger.Cli.Commands.Check
 import Hledger.Cli.Commands.Close
 import Hledger.Cli.Commands.Codes
 import Hledger.Cli.Commands.Commodities
+import Hledger.Cli.Commands.Demo
 import Hledger.Cli.Commands.Descriptions
 import Hledger.Cli.Commands.Diff
 import Hledger.Cli.Commands.Files
@@ -79,9 +88,7 @@ import Hledger.Cli.Commands.Notes
 import Hledger.Cli.Commands.Payees
 import Hledger.Cli.Commands.Prices
 import Hledger.Cli.Commands.Print
-import Hledger.Cli.Commands.Printunique
 import Hledger.Cli.Commands.Register
-import Hledger.Cli.Commands.Registermatch
 import Hledger.Cli.Commands.Rewrite
 import Hledger.Cli.Commands.Roi
 import Hledger.Cli.Commands.Stats
@@ -105,7 +112,8 @@ builtinCommands = [
   ,(closemode              , close)
   ,(codesmode              , codes)
   ,(commoditiesmode        , commodities)
-  ,(descriptionsmode        , descriptions)
+  ,(demomode               , demo)
+  ,(descriptionsmode       , descriptions)
   ,(diffmode               , diff)
   ,(filesmode              , files)
   ,(helpmode               , help')
@@ -115,8 +123,6 @@ builtinCommands = [
   ,(payeesmode             , payees)
   ,(pricesmode             , prices)
   ,(printmode              , print')
-  ,(printuniquemode        , printunique)
-  ,(registermatchmode      , registermatch)
   ,(registermode           , register)
   ,(rewritemode            , rewrite)
   ,(roimode                , roi)
@@ -125,6 +131,43 @@ builtinCommands = [
   ,(testmode               , testcmd)
   ]
 
+-- figlet -f FONTNAME hledger, then escape backslashes
+_banner_slant = drop 1 [""
+    -----------------------------------------80-------------------------------------
+  ,"    __    __         __               "
+  ,"   / /_  / /__  ____/ /___ ____  _____"
+  ,"  / __ \\/ / _ \\/ __  / __ `/ _ \\/ ___/"
+  ," / / / / /  __/ /_/ / /_/ /  __/ /    "
+  ,"/_/ /_/_/\\___/\\__,_/\\__, /\\___/_/     "
+  ,"                   /____/             "
+  ]
+
+_banner_smslant = drop 1 [""
+  ,"   __   __       __            "
+  ,"  / /  / /__ ___/ /__ ____ ____"
+  ," / _ \\/ / -_) _  / _ `/ -_) __/"
+  ,"/_//_/_/\\__/\\_,_/\\_, /\\__/_/   "
+  ,"                /___/          "
+  ]
+
+_banner_speed = drop 1 [""
+  ,"______ ______    _________                    "
+  ,"___  /____  /__________  /______ _____________"
+  ,"__  __ \\_  /_  _ \\  __  /__  __ `/  _ \\_  ___/"
+  ,"_  / / /  / /  __/ /_/ / _  /_/ //  __/  /    "
+  ,"/_/ /_//_/  \\___/\\__,_/  _\\__, / \\___//_/     "
+  ,"                         /____/               "
+  ]
+
+-- | Choose and apply an accent color for hledger output, if possible
+-- picking one that will contrast with the current terminal background colour.
+accent :: String -> String
+accent
+  | not useColorOnStdout          = id
+  | terminalIsLight == Just False = brightWhite
+  | terminalIsLight == Just True  = brightBlack
+  | otherwise                     = id
+
 -- | The commands list, showing command names, standard aliases,
 -- and short descriptions. This is modified at runtime, as follows:
 --
@@ -144,109 +187,151 @@ builtinCommands = [
 -- TODO: generate more of this automatically.
 -- 
 commandsList :: String -> [String] -> [String]
-commandsList progversion othercmds = [
-  -- keep synced with hledger.m4.md -> Commands, commands.m4 -->
+commandsList progversion othercmds =
+  map (bold'.accent) _banner_smslant ++ 
+  [
+  -- Keep the following synced with:
+  --  commands.m4
+  --  hledger.m4.md -> Commands
+  --  commandsFromCommandsList. Only commands should begin with space or plus.
    "-------------------------------------------------------------------------------"
   ,progversion
-  ,"Usage: hledger COMMAND [OPTIONS] [-- ADDONCMDOPTIONS]"
-  ,"Commands (+ addons found in $PATH):"
+  ,"Usage: hledger CMD [OPTS] [-- ADDONCMDOPTS]"
+  ,"Commands (builtins + addons):"
   ,""
-  ,"Data entry (these commands modify the journal file):"
-  ," add                      add transactions using guided prompts"
-  ,"+iadd                     add transactions using curses ui"
-  ," import                   add any new transactions from other files (eg csv)"
+  ,bold' "ENTERING DATA (add or edit transactions, updating the journal file)"
+  ," add                      add transactions using terminal prompts"
+  ,"+edit                     edit a subset of transactions"
+  ,"+iadd                     add transactions using a TUI"
+  ," import                   add new transactions from other files, eg CSV files"
   ,""
-  ,"Data management:"
+    -----------------------------------------80-------------------------------------
+  ,bold' "GENERATING DATA (generate entries to be added to the journal file)"
   ,"+autosync                 download/deduplicate/convert OFX data"
-  ," check                    check for various kinds of issue in the data"
-  ,"+check-fancyassertions    check more powerful balance assertions"
-  ,"+check-tagfiles           check file paths in tag values exist"
-  ," close                    generate balance-resetting transactions"
-  ," diff                     compare account transactions in two journal files"
+  ," close                    generate balance-zeroing/restoring transactions"
   ,"+interest                 generate interest transactions"
-  ," rewrite                  generate extra postings, similar to print --auto"
+  ,"+lots sell                generate a lot-selling transaction"
+  ," rewrite                  generate auto postings, like print --auto"
   ,"+stockquotes              download market prices from AlphaVantage"
   ,""
-  ,"Financial reports:"
+    -----------------------------------------80-------------------------------------
+  ,bold' "MANAGING DATA (error checking, file management, version control..)"
+  ," check                    check for various kinds of error in the data"
+  ,"+check-fancyassertions    check more powerful balance assertions"
+  ,"+check-tagfiles           check file paths in tag values exist"
+  ," diff                     compare account transactions in two journal files"
+  ," files                    show data files in use"
+  ,"+git                      simple version management with git"
+  ,"+pijul                    simple version management with pijul"
+  ,""
+    -----------------------------------------80-------------------------------------
+  ,bold' "FINANCIAL REPORTS (standard financial statements)"
   ," aregister (areg)         show transactions in a particular account"
   ," balancesheet (bs)        show assets, liabilities and net worth"
   ," balancesheetequity (bse) show assets, liabilities and equity"
   ," cashflow (cf)            show changes in liquid assets"
   ," incomestatement (is)     show revenues and expenses"
+  ,""
+    -----------------------------------------80-------------------------------------
+  ,bold' "VERSATILE REPORTS (more complex/versatile reporting commands)"
+  ," activity                 show a simple bar chart of posting counts per period"
+  ," balance (bal)            show balance changes, end balances, budgets, gains.."
+  ," bar                      show a balance report as a simple bar chart"
+  ,"+lots                     show a commodity's lots"
+  ,"+plot                     create charts from balance reports, in terminal or GUI"
+  ," print                    show transactions or export journal data"
+  ," register (reg)           show postings in one or more accounts & running total"
   ," roi                      show return on investments"
   ,""
-  ,"Low-level reports:"
+    -----------------------------------------80-------------------------------------
+  ,bold' "BASIC REPORTS (lists and stats)"
   ," accounts                 show account names"
-  ," activity                 show postings-per-interval bar charts"
-  ," balance (bal)            show balance changes/end balances/budgets in accounts"
   ," codes                    show transaction codes"
   ," commodities              show commodity/currency symbols"
-  ," descriptions             show unique transaction descriptions"
-  ," files                    show input file paths"
-  ," notes                    show unique note segments of transaction descriptions"
-  ," payees                   show unique payee segments of transaction descriptions"
-  ," prices                   show market price records"
-  ," print                    show transactions (journal entries)"
-  ," print-unique             show only transactions with unique descriptions"
-  ," register (reg)           show postings in one or more accounts & running total"
-  ," register-match           show a recent posting that best matches a description"
+  ," descriptions             show full transaction descriptions (payee and note)"
+  ," notes                    show note part of transaction descriptions"
+  ," payees                   show payee names"
+  ," prices                   show historical market prices"
   ," stats                    show journal statistics"
   ," tags                     show tag names"
   ," test                     run self tests"
   ,""
-  ,"Alternate user interfaces:"
+    -----------------------------------------80-------------------------------------
+  ,bold' "UIS (other user interfaces)"
   ,"+ui                       run terminal UI"
   ,"+web                      run web UI"
   ,""
-  ,"Other:"
-  ] ++
-  othercmds
-  ++
-  [""
-  ,"Help:"
-  ," (no arguments)           show this commands list"
-  ," -h                       show general flags"
-  ," COMMAND -h               show flags & docs for COMMAND"
-  ," help [MANUAL]            show hledger manuals in various formats"
+    -----------------------------------------80-------------------------------------
+  ,bold' "HELP (show help and docs)"
+  ," hledger                          show this commands list"
+  ," hledger -h                       show hledger's command-line help"
+  ," hledger CMD -h                   show CMD's command-line help and manual"
+  ," hledger help [-i|-m|-p] [TOPIC]  show hledger's manual with info, man, or pager"
+  ," hledger demo [DEMO] -- [ASCOPTS] show brief demos on various topics"
+  ," https://hledger.org              html manuals, tutorials, support.."
   ,""
+    -----------------------------------------80-------------------------------------
+  ,bold' "OTHER (more hledger-* addon commands found in PATH)"
   ]
--- edit                     open a text editor on some part of the journal
+  ++ map (' ':) (lines $ multicol 79 othercmds)
+  ++ [""]
 
+-- | Extract just the command names from the default commands list above,
+-- (the first word of lines between "Usage:" and "HELP" beginning with a space or plus sign),
+-- in the order they occur. With a true first argument, extracts only the addon command names.
+-- Needs to be kept synced with commandsList.
+commandsListExtractCommands :: Bool -> [String] -> [String]
+commandsListExtractCommands addonsonly l =
+  [ w | c:ws@(d:_) <- takeWhile (not . isInfixOf "HELP") $ dropWhile (not . isInfixOf "Usage:") l
+  , c `elem` '+':[' '|not addonsonly]
+  , isAlphaNum d
+  , not $ "://" `isInfixOf` ws
+  , let w:_ = words ws
+  ]
 
--- | All names and aliases of builtin commands.
+-- | Canonical names of all commands which have a slot in the commands list, in alphabetical order.
+-- These include the builtin commands and the known addon commands.
+knownCommands :: [String]
+knownCommands = nubSort . commandsListExtractCommands False $ commandsList progname []
+
+-- | Canonical names of the known addon commands which have a slot in the commands list,
+-- in alphabetical order.
+knownAddonCommands :: [String]
+knownAddonCommands = nubSort . commandsListExtractCommands True $ commandsList progname []
+
+-- | All names and aliases of the builtin commands.
 builtinCommandNames :: [String]
 builtinCommandNames = concatMap (modeNames . fst) builtinCommands
 
 -- | Look up a builtin command's mode and action by exact command name or alias. 
-findCommand :: String -> Maybe (Mode RawOpts, CliOpts -> Journal -> IO ()) 
-findCommand cmdname = find (elem cmdname . modeNames . fst) builtinCommands 
+findBuiltinCommand :: String -> Maybe (Mode RawOpts, CliOpts -> Journal -> IO ()) 
+findBuiltinCommand cmdname = find (elem cmdname . modeNames . fst) builtinCommands 
 
--- | Extract the command names from commandsList: the first word
--- of lines beginning with a space or + sign.
-commandsFromCommandsList :: [String] -> [String]
-commandsFromCommandsList s =
-  [w | c:l <- s, c `elem` [' ','+'], let w:_ = words l]
-
-knownCommands :: [String]
-knownCommands = sort . commandsFromCommandsList . drop 1 $ commandsList progname []  -- progname will not be seen
-
--- | Print the commands list, modifying the template above based on
--- the currently available addons. Missing addons will be removed, and
--- extra addons will be added under Misc.
+-- | Print the commands list, with a pager if appropriate, customising the
+-- commandsList template above with the given version string and the installed addons.
+-- Uninstalled known addons will be removed from the list,
+-- installed known addons will have the + prefix removed,
+-- and installed unknown addons will be added under Misc.
 printCommandsList :: String -> [String] -> IO ()
-printCommandsList progversion addonsFound =
-    putStr . unlines . concatMap adjustline $
-    commandsList progversion (map ('+':) unknownCommandsFound)
+printCommandsList progversion installedaddons =
+  seq (length $ dbg8 "uninstalledknownaddons" uninstalledknownaddons) $  -- for debug output
+  seq (length $ dbg8 "installedknownaddons"   installedknownaddons)   $
+  seq (length $ dbg8 "installedunknownaddons" installedunknownaddons) $
+  pager $ unlines $ map unplus $ filter (not.isuninstalledaddon) $
+  commandsList progversion installedunknownaddons
   where
-    commandsFound = map (' ':) builtinCommandNames ++ map ('+':) addonsFound
-    unknownCommandsFound = addonsFound \\ knownCommands
-
-    adjustline l         | " hledger " `isPrefixOf` l     = [l]
-    adjustline l@('+':_) | cmd `notElem` commandsFound = []
-      where
-        cmd = takeWhile (not . isSpace) l
-    adjustline l = [l]
-
+    knownaddons = knownAddonCommands
+    uninstalledknownaddons  = knownaddons \\ installedaddons
+    installedknownaddons    = knownaddons `intersect` installedaddons
+    installedunknownaddons  = installedaddons \\ knownaddons
+    unplus ('+':cs) = ' ':cs
+    unplus s = s
+    isuninstalledaddon =
+      \case
+        ('+':l) | cmd `notElem` installedaddons ->
+                  dbg9With (const $ "hiding uninstalled addon: "<>cmd) $
+                  True where cmd = takeWhile (not . isSpace) l
+        _ -> False
 
 -- The test command is defined here for easy access to other modules' tests.
 
@@ -292,8 +377,8 @@ tests_Commands = testGroup "Commands" [
         let
           ignoresourcepos j = j{jtxns=map (\t -> t{tsourcepos=nullsourcepos}) (jtxns j)}
           sameParse str1 str2 = do
-            j1 <- readJournal definputopts Nothing str1 >>= either error' (return . ignoresourcepos)  -- PARTIAL:
-            j2 <- readJournal definputopts Nothing str2 >>= either error' (return . ignoresourcepos)
+            j1 <- ignoresourcepos <$> readJournal' str1  -- PARTIAL:
+            j2 <- ignoresourcepos <$> readJournal' str2  -- PARTIAL:
             j1 @?= j2{jlastreadtime=jlastreadtime j1, jfiles=jfiles j1} --, jparsestate=jparsestate j1}
         sameParse
            ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" <>
@@ -310,19 +395,19 @@ tests_Commands = testGroup "Commands" [
            )
 
     ,testCase "preserves \"virtual\" posting type" $ do
-      j <- readJournal definputopts Nothing "apply account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return  -- PARTIAL:
+      j <- readJournal' "apply account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n"  -- PARTIAL:
       let p = head $ tpostings $ head $ jtxns j
       paccount p @?= "test:from"
       ptype p @?= VirtualPosting
     ]
 
   ,testCase "alias directive" $ do
-    j <- readJournal definputopts Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return  -- PARTIAL:
+    j <- readJournal' "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n"  -- PARTIAL:
     let p = head $ tpostings $ head $ jtxns j
     paccount p @?= "equity:draw:personal:food"
 
   ,testCase "Y default year directive" $ do
-    j <- readJournal definputopts Nothing defaultyear_journal_txt >>= either error' return  -- PARTIAL:
+    j <- readJournal' defaultyear_journal_txt  -- PARTIAL:
     tdate (head $ jtxns j) @?= fromGregorian 2009 1 1
 
   ,testCase "ledgerAccountNames" $
