@@ -25,13 +25,14 @@ import Data.Maybe (fromMaybe)
 import qualified Data.Text as T
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Builder as TB
+import Lucid as L hiding (value_)
 import System.Console.CmdArgs.Explicit (flagNone, flagReq)
 
 import Hledger
-import Hledger.Read.CsvReader (CSV, CsvRecord, printCSV)
+import Hledger.Read.CsvUtils (CSV, CsvRecord, printCSV)
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils
-import Text.Tabular.AsciiWide
+import Text.Tabular.AsciiWide hiding (render)
 
 aregistermode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Aregister.txt")
@@ -56,7 +57,8 @@ aregistermode = hledgerCommandMode
 #endif
       ++ " or $COLUMNS). -wN,M sets description width as well."
      )
-  ,outputFormatFlag ["txt","csv","json"]
+  ,flagNone ["align-all"] (setboolopt "align-all") "guarantee alignment across all lines (slower)"
+  ,outputFormatFlag ["txt","html","csv","json"]
   ,outputFileFlag
   ])
   [generalflagsgroup1]
@@ -70,10 +72,11 @@ aregister :: CliOpts -> Journal -> IO ()
 aregister opts@CliOpts{rawopts_=rawopts,reportspec_=rspec} j = do
   -- the first argument specifies the account, any remaining arguments are a filter query
   let help = "aregister needs an ACCTPAT argument to select an account"
-  (apat,querystring) <- case listofstringopt "args" rawopts of
+  (apat,querystr) <- case listofstringopt "args" rawopts of
       []     -> error' $ help <> ".\nPlease provide an account name or a (case-insensitive, infix, regexp) pattern."
       (a:as) -> return (a, map T.pack as)
   let
+    -- keep synced with accounts --find
     acct = fromMaybe (error' $ help <> ",\nbut " ++ show apat++" did not match any account.")   -- PARTIAL:
            . firstMatch $ journalAccountNamesDeclaredOrImplied j
     firstMatch = case toRegexCI $ T.pack apat of
@@ -87,7 +90,7 @@ aregister opts@CliOpts{rawopts_=rawopts,reportspec_=rspec} j = do
         depth_=Nothing
         -- always show historical balance
       , balanceaccum_= Historical
-      , querystring_ = querystring
+      , querystring_ = querystr
       }
     wd = whichDate ropts'
   -- and regenerate the ReportSpec, making sure to use the above
@@ -100,6 +103,7 @@ aregister opts@CliOpts{rawopts_=rawopts,reportspec_=rspec} j = do
              reverse items
     -- select renderer
     render | fmt=="txt"  = accountTransactionsReportAsText opts (_rsQuery rspec') thisacctq
+           | fmt=="html" = accountTransactionsReportAsHTML opts (_rsQuery rspec') thisacctq
            | fmt=="csv"  = printCSV . accountTransactionsReportAsCsv wd (_rsQuery rspec') thisacctq
            | fmt=="json" = toJsonText
            | otherwise   = error' $ unsupportedOutputFormatError fmt  -- PARTIAL:
@@ -124,25 +128,51 @@ accountTransactionsReportItemAsCsvRecord
     amt  = wbToText $ showMixedAmountB csvDisplay change
     bal  = wbToText $ showMixedAmountB csvDisplay balance
 
+-- | Render a register report as a HTML snippet.
+accountTransactionsReportAsHTML :: CliOpts -> Query -> Query -> AccountTransactionsReport -> TL.Text
+accountTransactionsReportAsHTML copts reportq thisacctq items =
+    L.renderText $ L.table_ (do L.thead_ (L.tr_ (do L.th_ "date"
+                                                    L.th_ "description"
+                                                    L.th_ "otheraccounts"
+                                                    L.th_ "change"
+                                                    L.th_ "balance"))
+                                L.tbody_ (mconcat (map (htmlRow copts reportq thisacctq) items)))
+
+-- | Render one account register report line item as a HTML table row snippet.
+htmlRow :: CliOpts -> Query -> Query -> AccountTransactionsReportItem -> L.Html ()
+htmlRow CliOpts{reportspec_=ReportSpec{_rsReportOpts=ropts}} reportq thisacctq
+    (t@Transaction{tdescription}, _, _issplit, otheracctsstr, amt, bal) =
+    L.tr_ (do (L.td_ . toHtml . show . transactionRegisterDate (whichDate ropts) reportq thisacctq) t
+              (L.td_ . toHtml) tdescription
+              (L.td_ . toHtml) otheracctsstr
+              -- piggy back on the oneLine display style for now.
+              (L.td_ . toHtml . wbUnpack . showMixedAmountB oneLine) amt
+              (L.td_ . toHtml . wbUnpack . showMixedAmountB oneLine) bal)
+
 -- | Render a register report as plain text suitable for console output.
 accountTransactionsReportAsText :: CliOpts -> Query -> Query -> AccountTransactionsReport -> TL.Text
 accountTransactionsReportAsText copts reportq thisacctq items = TB.toLazyText $
-    title <> TB.singleton '\n' <> lines
+    title <> TB.singleton '\n' <>
+    postingsOrTransactionsReportAsText alignAll copts itemAsText itemamt itembal items
   where
-    lines = foldMap (accountTransactionsReportItemAsText copts reportq thisacctq amtwidth balwidth) items
-    amtwidth = maximumStrict $ 12 : widths (map itemamt items)
-    balwidth = maximumStrict $ 12 : widths (map itembal items)
-    widths = map wbWidth . concatMap (showMixedAmountLinesB oneLine)
+    alignAll = boolopt "align-all" $ rawopts_ copts
+    itemAsText = accountTransactionsReportItemAsText copts reportq thisacctq
     itemamt (_,_,_,_,a,_) = a
     itembal (_,_,_,_,_,a) = a
 
     -- show a title indicating which account was picked, which can be confusing otherwise
-    title = maybe mempty (\s -> foldMap TB.fromText ["Transactions in ", s, " and subaccounts:"]) macct
+    title = maybe mempty (\s -> foldMap TB.fromText ["Transactions in ", s, " and subaccounts", qmsg, ":"]) macct
       where
         -- XXX temporary hack ? recover the account name from the query
         macct = case filterQuery queryIsAcct thisacctq of
                   Acct r -> Just . T.drop 1 . T.dropEnd 5 $ reString r  -- Acct "^JS:expenses(:|$)"
                   _      -> Nothing  -- shouldn't happen
+        -- show a hint in the title when results are restricted by an extra query (other than depth or date or date2)
+        qmsg = if hasextraquery then " (matching query)" else ""
+          where
+            hasextraquery =
+              length (querystring_ $ _rsReportOpts $ reportspec_ copts) > 1
+              && not (queryIsNull $ filterQuery (not.(\q->queryIsDepth q || queryIsDateOrDate2 q)) reportq)
 
 -- | Render one account register report line item as plain text. Layout is like so:
 -- @
@@ -156,11 +186,13 @@ accountTransactionsReportAsText copts reportq thisacctq items = TB.toLazyText $
 -- Returns a string which can be multi-line, eg if the running balance
 -- has multiple commodities.
 --
-accountTransactionsReportItemAsText :: CliOpts -> Query -> Query -> Int -> Int -> AccountTransactionsReportItem -> TB.Builder
+accountTransactionsReportItemAsText :: CliOpts -> Query -> Query -> Int -> Int
+                                    -> (AccountTransactionsReportItem, [WideBuilder], [WideBuilder])
+                                    -> TB.Builder
 accountTransactionsReportItemAsText
-  copts@CliOpts{reportspec_=ReportSpec{_rsReportOpts=ropts@ReportOpts{color_}}}
+  copts@CliOpts{reportspec_=ReportSpec{_rsReportOpts=ropts}}
   reportq thisacctq preferredamtwidth preferredbalwidth
-  (t@Transaction{tdescription}, _, _issplit, otheracctsstr, change, balance) =
+  ((t@Transaction{tdescription}, _, _issplit, otheracctsstr, _, _), amt, bal) =
     -- Transaction -- the transaction, unmodified
     -- Transaction -- the transaction, as seen from the current account
     -- Bool        -- is this a split (more than one posting to other accounts) ?
@@ -182,8 +214,8 @@ accountTransactionsReportItemAsText
       ]
     spacerCell  = Cell BottomLeft [WideBuilder (TB.singleton ' ') 1]
     spacerCell2 = Cell BottomLeft [WideBuilder (TB.fromString "  ") 2]
-    pad fullwidth amt = WideBuilder (TB.fromText $ T.replicate w " ") w <> amt
-      where w = fullwidth - wbWidth amt
+    pad fullwidth amt1 = WideBuilder (TB.fromText $ T.replicate w " ") w <> amt1
+      where w = fullwidth - wbWidth amt1
     -- calculate widths
     (totalwidth,mdescwidth) = registerWidthsFromOpts copts
     (datewidth, date) = (10, showDate $ transactionRegisterDate wd reportq thisacctq t)
@@ -206,9 +238,6 @@ accountTransactionsReportItemAsText
     -- gather content
     accts = -- T.unpack $ elideAccountName acctwidth $ T.pack
             otheracctsstr
-    amt = showamt change
-    bal = showamt balance
-    showamt = showMixedAmountLinesB noPrice{displayColour=color_}
 
 -- tests
 
