@@ -15,26 +15,25 @@ module Hledger.Cli.Commands.Print (
 )
 where
 
-import Data.Text (Text)
 import Data.List (intersperse)
 import qualified Data.Text as T
-import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Builder as TB
 import Lens.Micro ((^.), _Just, has)
 import System.Console.CmdArgs.Explicit
 
 import Hledger
-import Hledger.Read.CsvReader (CSV, printCSV)
+import Hledger.Read.CsvUtils (CSV, printCSV)
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils
+import System.Exit (exitFailure)
 
 
 printmode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Print.txt")
-  ([let arg = "STR" in
+  ([let arg = "DESC" in
    flagReq  ["match","m"] (\s opts -> Right $ setopt "match" s opts) arg
-    ("show the transaction whose description is most similar to "++arg++", and is most recent")
+    ("fuzzy search for one recent transaction with description closest to "++arg)
   ,flagNone ["explicit","x"] (setboolopt "explicit")
     "show all amounts explicitly"
   ,flagNone ["show-costs"] (setboolopt "show-costs")
@@ -60,11 +59,16 @@ print' opts j = do
   let j' = journalMapPostingAmounts mixedAmountSetFullPrecision j
   case maybestringopt "match" $ rawopts_ opts of
     Nothing   -> printEntries opts j'
-    Just desc -> printMatch opts j' $ T.pack $ dbg1 "finding best match for description" desc
+    Just desc -> 
+      -- match mode, prints one recent transaction most similar to given description
+      -- XXX should match similarly to register --match
+      case journalSimilarTransaction opts j' (dbg1 "finding best match for description" $ T.pack desc) of
+        Just t  -> printEntries opts j'{jtxns=[t]}
+        Nothing -> putStrLn "no matches found." >> exitFailure
 
 printEntries :: CliOpts -> Journal -> IO ()
 printEntries opts@CliOpts{reportspec_=rspec} j =
-    writeOutputLazyText opts . render $ entriesReport rspec j
+  writeOutputLazyText opts . render $ entriesReport rspec j
   where
     fmt = outputFormatFromOpts opts
     render | fmt=="txt"  = entriesReportAsText opts
@@ -75,11 +79,13 @@ printEntries opts@CliOpts{reportspec_=rspec} j =
 
 entriesReportAsText :: CliOpts -> EntriesReport -> TL.Text
 entriesReportAsText opts =
-    TB.toLazyText . foldMap (TB.fromText . showTransaction . maybeStripPrices . whichtxn)
+    TB.toLazyText . foldMap (TB.fromText . showTransaction . whichtxn)
   where
     whichtxn
       -- With -x, use the fully-inferred txn with all amounts & txn prices explicit.
       | boolopt "explicit" (rawopts_ opts) = id
+      -- With --show-costs, make txn prices explicit.
+      | opts ^. infer_costs = id
       -- Or also, if any of -B/-V/-X/--value are active.
       -- Because of #551, and because of print -V valuing only one
       -- posting when there's an implicit txn price.
@@ -87,11 +93,6 @@ entriesReportAsText opts =
       | has (value . _Just) opts = id
       -- By default, use the original as-written-in-the-journal txn.
       | otherwise = originalTransaction
-    maybeStripPrices
-      -- Strip prices when inferring equity, unless the show_costs_  is set
-      | opts ^. infer_equity && not (opts ^. show_costs) =
-          transactionTransformPostings postingStripPrices
-      | otherwise = id
 
 -- Replace this transaction's postings with the original postings if any, but keep the
 -- current possibly rewritten account names, and the inferred values of any auto postings
@@ -104,7 +105,7 @@ originalPostingPreservingAccount p = orig
     , pamount = pamount $ if isGenerated then p else orig }
   where
     orig = originalPosting p
-    isGenerated = "generated-posting" `elem` map fst (ptags p)
+    isGenerated = "_generated-posting" `elem` map fst (ptags p)
 
 -- XXX
 -- tests_showTransactions = [
@@ -171,13 +172,13 @@ entriesReportAsCsv txns =
 -- The txnidx field (transaction index) allows postings to be grouped back into transactions.
 transactionToCSV :: Transaction -> CSV
 transactionToCSV t =
-  map (\p -> T.pack (show idx):date:date2:status:code:description:comment:p)
+  map (\p -> T.pack (show idx):d:d2:status:code:description:comment:p)
    (concatMap postingToCSV $ tpostings t)
   where
     idx = tindex t
     description = tdescription t
-    date = showDate (tdate t)
-    date2 = maybe "" showDate $ tdate2 t
+    d = showDate (tdate t)
+    d2 = maybe "" showDate $ tdate2 t
     status = T.pack . show $ tstatus t
     code = tcode t
     comment = T.strip $ tcomment t
@@ -189,22 +190,12 @@ postingToCSV p =
     -- separators and prices
     let a_ = amountStripPrices a{acommodity=""} in
     let showamt = wbToText . showAmountB csvDisplay in
-    let amount = showamt a_ in
+    let amt = showamt a_ in
     let credit = if q < 0 then showamt $ negate a_ else "" in
     let debit  = if q >= 0 then showamt a_ else "" in
-    [account, amount, c, credit, debit, status, comment])
+    [account, amt, c, credit, debit, status, comment])
     . amounts $ pamount p
   where
     status = T.pack . show $ pstatus p
     account = showAccountName Nothing (ptype p) (paccount p)
     comment = T.strip $ pcomment p
-
--- --match
-
--- | Print the transaction most closely and recently matching a description
--- (and the query, if any).
-printMatch :: CliOpts -> Journal -> Text -> IO ()
-printMatch opts j desc = do
-  case journalSimilarTransaction opts j desc of
-    Nothing -> putStrLn "no matches found."
-    Just t  -> T.putStr $ showTransaction t
