@@ -22,14 +22,19 @@ import Data.Default (def)
 import Data.Maybe (fromMaybe, isJust)
 import qualified Data.Text as T
 import qualified Data.Text.Lazy as TL
+import qualified Data.Text.Lazy.IO as TL
 import qualified Data.Text.Lazy.Builder as TB
 import System.Console.CmdArgs.Explicit (flagNone, flagReq)
 
-import Hledger
-import Hledger.Read.CsvReader (CSV, CsvRecord, printCSV)
+import Hledger hiding (per)
+import Hledger.Read.CsvUtils (CSV, CsvRecord, printCSV)
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils
-import Text.Tabular.AsciiWide
+import Text.Tabular.AsciiWide hiding (render)
+import Data.List (sortBy)
+import Data.Char (toUpper)
+import Data.List.Extra (intersect)
+import System.Exit (exitFailure)
 
 registermode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Register.txt")
@@ -39,6 +44,9 @@ registermode = hledgerCommandMode
      "show historical running total/balance (includes postings before report start date)\n "
   ,flagNone ["average","A"] (setboolopt "average")
      "show running average of posting amounts instead of total (implies --empty)"
+  ,let arg = "DESC" in
+   flagReq  ["match","m"] (\s opts -> Right $ setopt "match" s opts) arg
+    ("fuzzy search for one recent posting with description closest to "++arg)
   ,flagNone ["related","r"] (setboolopt "related") "show postings' siblings instead"
   ,flagNone ["invert"] (setboolopt "invert") "display all amounts with reversed sign"
   ,flagReq  ["width","w"] (\s opts -> Right $ setopt "width" s opts) "N"
@@ -50,6 +58,7 @@ registermode = hledgerCommandMode
 #endif
       ++ " or $COLUMNS). -wN,M sets description width as well."
      )
+  ,flagNone ["align-all"] (setboolopt "align-all") "guarantee alignment across all lines (slower)"
   ,outputFormatFlag ["txt","csv","json"]
   ,outputFileFlag
   ])
@@ -59,14 +68,28 @@ registermode = hledgerCommandMode
 
 -- | Print a (posting) register report.
 register :: CliOpts -> Journal -> IO ()
-register opts@CliOpts{reportspec_=rspec} j =
-    writeOutputLazyText opts . render $ postingsReport rspec j
+register opts@CliOpts{rawopts_=rawopts, reportspec_=rspec} j
+  -- match mode, print one recent posting most similar to given description, if any
+  -- XXX should match similarly to print --match
+  | Just desc <- maybestringopt "match" rawopts = do
+      let ps = [p | (_,_,_,p,_) <- rpt]
+      case similarPosting ps desc of
+        Nothing -> putStrLn "no matches found." >> exitFailure
+        Just p  -> TL.putStr $ postingsReportAsText opts [pri]
+                  where pri = (Just (postingDate p)
+                              ,Nothing
+                              ,tdescription <$> ptransaction p
+                              ,p
+                              ,nullmixedamt)
+  -- normal register report, list postings
+  | otherwise = writeOutputLazyText opts $ render rpt
   where
-    fmt = outputFormatFromOpts opts
+    rpt = postingsReport rspec j
     render | fmt=="txt"  = postingsReportAsText opts
            | fmt=="csv"  = printCSV . postingsReportAsCsv
            | fmt=="json" = toJsonText
            | otherwise   = error' $ unsupportedOutputFormatError fmt  -- PARTIAL:
+      where fmt = outputFormatFromOpts opts
 
 postingsReportAsCsv :: PostingsReport -> CSV
 postingsReportAsCsv is =
@@ -93,12 +116,10 @@ postingsReportItemAsCsvRecord (_, _, _, p, b) = [idx,date,code,desc,acct,amt,bal
 
 -- | Render a register report as plain text suitable for console output.
 postingsReportAsText :: CliOpts -> PostingsReport -> TL.Text
-postingsReportAsText opts items = TB.toLazyText lines
+postingsReportAsText opts = TB.toLazyText .
+    postingsOrTransactionsReportAsText alignAll opts (postingsReportItemAsText opts) itemamt itembal
   where
-    lines = foldMap (postingsReportItemAsText opts amtwidth balwidth) items
-    amtwidth = maximumStrict $ 12 : widths (map itemamt items)
-    balwidth = maximumStrict $ 12 : widths (map itembal items)
-    widths = map wbWidth . concatMap (showMixedAmountLinesB oneLine)
+    alignAll = boolopt "align-all" $ rawopts_ opts
     itemamt (_,_,_,Posting{pamount=a},_) = a
     itembal (_,_,_,_,a) = a
 
@@ -126,8 +147,10 @@ postingsReportAsText opts items = TB.toLazyText lines
 --
 -- Also returns the natural width (without padding) of the amount and balance
 -- fields.
-postingsReportItemAsText :: CliOpts -> Int -> Int -> PostingsReportItem -> TB.Builder
-postingsReportItemAsText opts preferredamtwidth preferredbalwidth (mdate, mperiod, mdesc, p, b) =
+postingsReportItemAsText :: CliOpts -> Int -> Int
+                         -> (PostingsReportItem, [WideBuilder], [WideBuilder])
+                         -> TB.Builder
+postingsReportItemAsText opts preferredamtwidth preferredbalwidth ((mdate, mperiod, mdesc, p, _), amt, bal) =
     table <> TB.singleton '\n'
   where
     table = renderRowB def{tableBorders=False, borderSpaces=False} . Group NoLine $ map Header
@@ -143,14 +166,14 @@ postingsReportItemAsText opts preferredamtwidth preferredbalwidth (mdate, mperio
       ]
     spacerCell  = Cell BottomLeft [WideBuilder (TB.singleton ' ') 1]
     spacerCell2 = Cell BottomLeft [WideBuilder (TB.fromString "  ") 2]
-    pad fullwidth amt = WideBuilder (TB.fromText $ T.replicate w " ") w <> amt
-      where w = fullwidth - wbWidth amt
+    pad fullwidth amt' = WideBuilder (TB.fromText $ T.replicate w " ") w <> amt'
+      where w = fullwidth - wbWidth amt'
     -- calculate widths
     (totalwidth,mdescwidth) = registerWidthsFromOpts opts
     datewidth = maybe 10 periodTextWidth mperiod
     date = case mperiod of
-             Just period -> if isJust mdate then showPeriod period else ""
-             Nothing     -> maybe "" showDate mdate
+             Just per -> if isJust mdate then showPeriod per else ""
+             Nothing  -> maybe "" showDate mdate
     (amtwidth, balwidth)
       | shortfall <= 0 = (preferredamtwidth, preferredbalwidth)
       | otherwise      = (adjustedamtwidth, adjustedbalwidth)
@@ -177,9 +200,55 @@ postingsReportItemAsText opts preferredamtwidth preferredbalwidth (mdate, mperio
             BalancedVirtualPosting -> (wrap "[" "]", acctwidth-2)
             VirtualPosting         -> (wrap "(" ")", acctwidth-2)
             _                      -> (id,acctwidth)
-    amt = showamt $ pamount p
-    bal = showamt b
-    showamt = showMixedAmountLinesB oneLine{displayColour=color_ . _rsReportOpts $ reportspec_ opts}
+
+-- for register --match:
+
+-- Identify the closest recent match for this description in the given date-sorted postings.
+similarPosting :: [Posting] -> String -> Maybe Posting
+similarPosting ps desc =
+  let matches =
+          sortBy compareRelevanceAndRecency
+                     $ filter ((> threshold).fst)
+                     [(maybe 0 (\t -> compareDescriptions desc (T.unpack $ tdescription t)) (ptransaction p), p) | p <- ps]
+              where
+                compareRelevanceAndRecency (n1,p1) (n2,p2) = compare (n2,postingDate p2) (n1,postingDate p1)
+                threshold = 0
+  in case matches of []  -> Nothing
+                     m:_ -> Just $ snd m
+
+-- -- Identify the closest recent match for this description in past transactions.
+-- similarTransaction :: Journal -> Query -> String -> Maybe Transaction
+-- similarTransaction j q desc =
+--   case historymatches = transactionsSimilarTo j q desc of
+--     ((,t):_) = Just t
+--     []       = Nothing
+
+compareDescriptions :: String -> String -> Double
+compareDescriptions s t = compareStrings s' t'
+    where s' = simplify s
+          t' = simplify t
+          simplify = filter (not . (`elem` ("0123456789"::String)))
+
+-- | Return a similarity measure, from 0 to 1, for two strings.
+-- This is Simon White's letter pairs algorithm from
+-- http://www.catalysoft.com/articles/StrikeAMatch.html
+-- with a modification for short strings.
+compareStrings :: String -> String -> Double
+compareStrings "" "" = 1
+compareStrings [_] "" = 0
+compareStrings "" [_] = 0
+compareStrings [a] [b] = if toUpper a == toUpper b then 1 else 0
+compareStrings s1 s2 = 2.0 * fromIntegral i / fromIntegral u
+    where
+      i = length $ intersect pairs1 pairs2
+      u = length pairs1 + length pairs2
+      pairs1 = wordLetterPairs $ uppercase s1
+      pairs2 = wordLetterPairs $ uppercase s2
+
+wordLetterPairs = concatMap letterPairs . words
+
+letterPairs (a:b:rest) = [a,b] : letterPairs (b:rest)
+letterPairs _ = []
 
 -- tests
 
