@@ -1,6 +1,6 @@
 {-|
 hledger - a ledger-compatible accounting tool.
-Copyright (c) 2007-2011 Simon Michael <simon@joyful.com>
+Copyright (c) 2007-2022 Simon Michael <simon@joyful.com>
 Released under GPL version 3 or later.
 
 hledger is a partial haskell clone of John Wiegley's "ledger".  It
@@ -40,7 +40,7 @@ etc.
 
 module Hledger.Cli.Main where
 
-import Data.Char (isDigit)
+import Control.Monad (when)
 import Data.List
 import Safe
 import qualified System.Console.CmdArgs.Explicit as C
@@ -97,7 +97,9 @@ mainmode addons = defMode {
 -- | Let's go!
 main :: IO ()
 main = do
-  progstarttime <- getPOSIXTime
+  starttime <- getPOSIXTime
+  -- try to encourage user's $PAGER to properly display ANSI
+  when useColorOnStdout setupPager
 
   -- Choose and run the appropriate internal or external command based
   -- on the raw command-line arguments, cmdarg's interpretation of
@@ -132,7 +134,7 @@ main = do
 
   -- parse arguments with cmdargs
   opts' <- argsToCliOpts args addons
-  let opts = opts'{progstarttime_=progstarttime}
+  let opts = opts'{progstarttime_=starttime}
 
   -- select an action and run it.
   let
@@ -141,15 +143,15 @@ main = do
     isExternalCommand    = not (null cmd) && cmd `elem` addons -- probably
     isBadCommand         = not (null rawcmd) && null cmd
     hasVersion           = ("--version" `elem`)
-    printUsage           = putStr $ showModeUsage $ mainmode addons
+    printUsage           = pager $ showModeUsage $ mainmode addons
     badCommandError      = error' ("command "++rawcmd++" is not recognized, run with no command to see a list") >> exitFailure  -- PARTIAL:
-    hasHelpFlag args     = any (`elem` args) ["-h","--help"]
-    hasManFlag args      = any (`elem` args) ["--man"]
-    hasInfoFlag args     = any (`elem` args) ["--info"]
-    f `orShowHelp` mode
-      | hasHelpFlag args = putStr $ showModeUsage mode
-      | hasInfoFlag args = runInfoForTopic "hledger" (headMay $ modeNames mode)
-      | hasManFlag args  = runManForTopic "hledger" (headMay $ modeNames mode)
+    hasHelpFlag args1     = any (`elem` args1) ["-h","--help"]
+    hasManFlag args1      = (`elem` args1) "--man"
+    hasInfoFlag args1     = (`elem` args1) "--info"
+    f `orShowHelp` mode1
+      | hasHelpFlag args = pager $ showModeUsage mode1
+      | hasInfoFlag args = runInfoForTopic "hledger" (headMay $ modeNames mode1)
+      | hasManFlag args  = runManForTopic "hledger" (headMay $ modeNames mode1)
       | otherwise        = f
       -- where
       --   lastdocflag
@@ -172,13 +174,13 @@ main = do
       | not (isExternalCommand || hasHelpFlag args || hasInfoFlag args || hasManFlag args)
         && (hasVersion args) --  || (hasVersion argsaftercmd && isInternalCommand))
                                  = putStrLn prognameandversion
-      -- \| (null externalcmd) && "binary-filename" `inRawOpts` rawopts = putStrLn $ binaryfilename progname
+      -- \| (null externalcmd) && boolopt "binary-filename" rawopts = putStrLn $ binaryfilename progname
       -- \| "--browse-args" `elem` args     = System.Console.CmdArgs.Helper.execute "cmdargs-browser" mainmode' args >>= (putStr . show)
       | isNullCommand            = dbgIO "" "no command, showing commands list" >> printCommandsList prognameandversion addons
       | isBadCommand             = badCommandError
 
       -- builtin commands
-      | Just (cmdmode, cmdaction) <- findCommand cmd =
+      | Just (cmdmode, cmdaction) <- findBuiltinCommand cmd =
         (case True of
            -- these commands should not require or read the journal
           _ | cmd `elem` ["test","help"] -> cmdaction opts journallesserror
@@ -220,24 +222,18 @@ argsToCliOpts args addons = do
 -- | A hacky workaround for cmdargs not accepting flags before the
 -- subcommand name: try to detect and move such flags after the
 -- command.  This allows the user to put them in either position.
--- The order of options is not preserved, but this should be ok.
+-- The order of options is not preserved, but that should be ok.
 --
 -- Since we're not parsing flags as precisely as cmdargs here, this is
 -- imperfect. We make a decent effort to:
 -- - move all no-argument help/input/report flags
 -- - move all required-argument help/input/report flags along with their values, space-separated or not
+-- - ensure --debug has an argument (because.. "or this all goes to hell")
 -- - not confuse things further or cause misleading errors.
 moveFlagsAfterCommand :: [String] -> [String]
 moveFlagsAfterCommand args = moveArgs $ ensureDebugHasArg args
   where
-    -- quickly! make sure --debug has a numeric argument, or this all goes to hell
-    ensureDebugHasArg as =
-      case break (=="--debug") as of
-       (bs,"--debug":c:cs) | null c || not (all isDigit c) -> bs++"--debug=1":c:cs
-       (bs,["--debug"])                                    -> bs++["--debug=1"]
-       _                                                   -> as
-
-    moveArgs args = insertFlagsAfterCommand $ moveArgs' (args, [])
+    moveArgs args1 = insertFlagsAfterCommand $ moveArgs' (args1, [])
       where
         -- -f FILE ..., --alias ALIAS ...
         moveArgs' ((f:v:a:as), flags) | isMovableReqArgFlag f, isValue v       = moveArgs' (a:as, flags ++ [f,v])
@@ -251,7 +247,7 @@ moveFlagsAfterCommand args = moveArgs $ ensureDebugHasArg args
         moveArgs' (as, flags) = (as, flags)
 
         insertFlagsAfterCommand ([],           flags) = flags
-        insertFlagsAfterCommand (command:args, flags) = [command] ++ flags ++ args
+        insertFlagsAfterCommand (command1:args2, flags) = [command1] ++ flags ++ args2
 
 isMovableNoArgFlag a  = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` optargflagstomove ++ noargflagstomove
 
