@@ -13,7 +13,7 @@ module Hledger.Cli.Version (
 )
 where
 
-import GitHash (GitInfo, giHash, giCommitDate)
+import GitHash (GitInfo, giHash, giCommitDate)  -- giDirty
 import System.Info (os, arch)
 import Data.List (intercalate)
 import Data.Maybe (fromMaybe)
@@ -42,7 +42,9 @@ progname = "hledger"
 -- 
 -- * Program name, OS and architecture are always shown.
 -- * The package version is always shown.
--- * If there is git info at build time, the latest commit hash and commit date are shown.
+-- * If there is git info at build time, the latest commit hash and commit date are shown,
+--   and (TODO, requires githash to use -uno for giDirty):
+--   if the working copy has uncommitted changes a + sign is appended.
 -- * (TODO, requires adding --match support to githash:
 --   If there are tags matching THISPKG-[0-9]*, the latest one is used to calculate patch level
 --   (number of commits since tag), and if non-zero, it and the branch name are shown.)
@@ -57,38 +59,42 @@ progname = "hledger"
 -- so that must not be overridden by a log.date git config variable.
 --
 versionStringWith :: Either String GitInfo -> ProgramName -> PackageVersion -> VersionString
-versionStringWith egitinfo progname packageversion =
-  concat [ progname , " " , version , ", " , os' , "-" , arch ]
+versionStringWith egitinfo prognam packagever =
+  concat [ prognam , " " , version , ", " , os' , "-" , arch ]
   where
     os' | os == "darwin"  = "mac"
         | os == "mingw32" = "windows"
         | otherwise       = os
     version = case egitinfo of
-      Left _err     -> packageversion
-      Right gitinfo -> intercalate "-" [packageversion , hash, date]
-        where
-          hash = 'g' : take 9 (giHash gitinfo)  -- like git describe
-          date = concat [year,mm,dd]
-            where 
-              -- XXX PARTIAL: depends on git log's date format, which by default
-              -- is --date=default ("similar to --date=rfc2822"), but could be
-              -- overridden by a log.date config variable in repo or user git config.
-              _weekday:mon:day:_localtime:year:_offset:_ = words $ giCommitDate gitinfo
-              mm = fromMaybe mon $ lookup mon $ [
-                 ("Jan","01")
-                ,("Feb","02")
-                ,("Mar","03")
-                ,("Apr","04")
-                ,("May","05")
-                ,("Jun","06")
-                ,("Jul","07")
-                ,("Aug","08")
-                ,("Sep","09")
-                ,("Oct","10")
-                ,("Nov","11")
-                ,("Dec","12")
-                ]
-              dd = (if length day < 2 then ('0':) else id) day
+      Left _err     -> packagever
+      Right gitinfo -> 
+        case words $ giCommitDate gitinfo of
+          -- git log's date format is normally --date=default ("similar to --date=rfc2822")
+          _weekday:mon:day:_localtime:year:_offset:_ ->
+            intercalate "-" $ [packagever, hash, date]
+              -- ++ ["+" | giDirty gitinfo]
+              --   XXX giDirty is wrong when repo shows untracked files by default, skip it for now
+              where
+                hash = 'g' : take 9 (giHash gitinfo)  -- like git describe
+                date = concat [year,mm,dd]
+                  where 
+                    mm = fromMaybe mon $ lookup mon $ [
+                      ("Jan","01")
+                      ,("Feb","02")
+                      ,("Mar","03")
+                      ,("Apr","04")
+                      ,("May","05")
+                      ,("Jun","06")
+                      ,("Jul","07")
+                      ,("Aug","08")
+                      ,("Sep","09")
+                      ,("Oct","10")
+                      ,("Nov","11")
+                      ,("Dec","12")
+                      ]
+                    dd = (if length day < 2 then ('0':) else id) day
+          -- but could be overridden by a log.date config variable in repo or user git config
+          _ -> packageversion
 
 -- -- | Given a program name, return a precise platform-specific executable
 -- -- name suitable for naming downloadable binaries.  Can raise an error if
